package com.sf.common.utils.openssl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;

import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

public class AES256Util {
    private static final String ALGORITHM = "AES/ECB/PKCS5Padding";

    public static Cipher initAESCipher(String key, int cipherMode) throws Exception {
        Cipher cipher = Cipher.getInstance(ALGORITHM);
        SecretKeySpec secretKeySpec = new SecretKeySpec(key.getBytes(StandardCharsets.UTF_8), "AES");
        cipher.init(cipherMode, secretKeySpec);

        return cipher;
    }

    /**
     * 加密文件
     *
     * @return
     */
    public static InputStream encryptFile(String key, InputStream inputStream) throws Exception {
        Cipher cipher = initAESCipher(key, Cipher.ENCRYPT_MODE);
        CipherInputStream cipherInputStream = new CipherInputStream(inputStream, cipher);

        return cipherInputStream;
    }

    /**
     * 解密文件
     *
     * @param encryptedFile
     * @param sourceFile
     */
    public static String decryptFile(String key, File encryptedFile, File sourceFile) {
        InputStream inputStream = null;
        OutputStream outputStream = null;
        CipherOutputStream cipherOutputStream = null;
        try {
            inputStream = new FileInputStream(encryptedFile);
            outputStream = new FileOutputStream(sourceFile);
            Cipher cipher = initAESCipher(key, Cipher.DECRYPT_MODE);
            cipherOutputStream = new CipherOutputStream(outputStream, cipher);
            byte[] buffer = new byte[1024];
            int n = -1;
            int count = 0;
            while ((n = inputStream.read(buffer)) != -1) {
                count += n;
                cipherOutputStream.write(buffer, 0, n);
                cipherOutputStream.flush();
            }

        } catch (Exception e) {
            return e.getLocalizedMessage();
        }
        if (cipherOutputStream != null) {
            try {
                cipherOutputStream.close();
            } catch (IOException e) {
                return e.getLocalizedMessage();
            }
        }
        if (outputStream != null) {
            try {
                outputStream.close();
            } catch (IOException e) {
                return e.getLocalizedMessage();
            }
        }
        if (inputStream != null) {
            try {
                inputStream.close();
            } catch (IOException e) {
                return e.getLocalizedMessage();
            }
        }

        return "";
    }

//    public static void main(String[] args) throws Exception {
//        File sourceFile = new File("/Users/a1234/Downloads/dist.zip");
//        InputStream encryptInputStream = encryptFile("efghijklmnopqrstuvwxyz0123456789", new FileInputStream(sourceFile));
//        OutputStream outputStream = new FileOutputStream(new File("/Users/a1234/Downloads/x.zip"));
//        byte[] buffer = new byte[1024];
//        int n = -1;
//        int count = 0;
//        while ((n = encryptInputStream.read(buffer)) != -1) {
//            count += n;
//            outputStream.write(buffer, 0, n);
//        }
//        outputStream.flush();
//        System.out.println(count);
//
//        decryptFile("efghijklmnopqrstuvwxyz0123456789", new File("/Users/a1234/Downloads/x.zip"), new File("/Users/a1234/Downloads/y.zip"));
//    }
}

